//////////////////////////////////////////////////
// This file contains the main class Main Menu, //
// as well as other secondary classes required  //
// to render the example menu				    //
//////////////////////////////////////////////////
import java.applet.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import com.sybase.dpb.*;
import com.softtreetech.scheduler.*;

public class MainMenu extends JApplet {
	private Frame appletFrame;
	private JScrollPane treeView;
	private static JTree tree;
	private static DefaultMutableTreeNode root;
    private static JTextArea jobDescPane;
	private JPanel 		buttonPane;
	private JButton 	disableButton, enableButton,
						logButton, runButton,
						createButton, deleteButton,
						modifyButton, describeButton,
						commandButton, scriptButton, sampleButton;
	
    private static boolean DEBUG = true;
	// Define global variables for JDPB_Connection and jdpb_24x7 classes:
    private static JDPB_Connection myConnection = new JDPB_Connection();
    private static jdpb_24x7 myRemoteControl = null;
    private static String connectString = new String("");
    private static Integer portNo = new Integer(1096);
    private static String hostName = new String("LocalHost");
    private static String userID = new String("scott");
    private static String userPass = new String("tiger");
    private static String serial = new String(""); 	// "" Serial Number is good for trial version 
													// only. You must serial number assigned to 
													// your license when using 24x7 Java API in
													// development production environment 


    private void buildTreeView() {
        ///////////////////////////////////////////
        // Create job nodes.
        ///////////////////////////////////////////
        
		// Get login parameters
		if (!getLoginInfo()) // Terminate applet
		{
			stop();
			return;// Terminate applet
		}
		
		if (DEBUG) System.out.println( "Connecting..." );
		
        try {
            myConnection.connect(hostName, 
                                 portNo.intValue(), userID, 
                                 serial, connectString);
        } 
		catch(java.lang.Throwable e) {
            showMessage( "Exception: " + e.getMessage() );
			// Terminate applet
			stop();
			return;// Terminate applet
        }  // end catch 
        
		
		if (DEBUG) System.out.println( "Connected. Starting new session..." );
		
        //    Instantiate the 24x7 Remote Control proxy class, passing the JDPB_Connection instance to 
        //    the constructor:
        myRemoteControl = new jdpb_24x7(myConnection);	
	
		//    Call 24x7 Remote Control methods from the Java application:
        try {
            // Open new session
            String terminal = new String("");

            Integer ReturnValue = 
                myRemoteControl.opensession( terminal, userID, userPass, serial );

            if (ReturnValue.intValue() == 1) {
                // Ok. Connection opened, session started successfully,
                // now we can do the scheduling things...
				if (DEBUG) System.out.println( "Session started. Getting job list..." );
				
				StringHolder jobList = new StringHolder("");
                ReturnValue =  myRemoteControl.getjoblistex( jobList, new Boolean(false), "DISABLED,FOLDER_NAME" );
        		if (DEBUG) System.out.println( "Done" );
				
                if (ReturnValue.intValue() == 1) {
                    // create nodes
					root = new DefaultMutableTreeNode(hostName);
                    createNodes(root, (String)jobList.getValue());
                } // end if
            }
			else {
				if (DEBUG) System.out.print( myRemoteControl.getLasterror());
				//    Disconnect from the 24x7 Scheduler server:
		        myConnection.disconnect();
				if (DEBUG) System.out.println( "Disconnected." );
				// Terminate applet
				stop();
				return;// Terminate applet
            } // end if
        
            if (ReturnValue.intValue() < 0) {
                // Handle the error
                showMessage(myRemoteControl.getLasterror());
            } // end if
        } //end try
        catch (com.sybase.dpb.RemoteException e) {
           showMessage( "RemoteException exception: " + e.getMessage() );
        }  // end catch 
		
        //Create a tree that allows one selection at a time.
        tree = new JTree(root);
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);

		tree.putClientProperty("JTree.lineStyle", "Angled");
		tree.setShowsRootHandles(true);
		
		
		//Enable tool tips.
        ToolTipManager.sharedInstance().registerComponent(tree);

        /*
         * Set the icon for leaf nodes.
         * Note: In the Swing 1.0.x release, we used
         * swing.plaf.basic.BasicTreeCellRenderer.
         */
        tree.setCellRenderer(new MyRenderer());
		
		
        //Listen for when the selection changes.
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                   tree.getLastSelectedPathComponent();

                if (node == null) return;

                Object nodeInfo = node.getUserObject();
                if (node.isLeaf()) {
                    jobInfo job = (jobInfo)nodeInfo;
                    displayDesc(job.jobID);
                    if (DEBUG) System.out.print(job.jobDesc + "\n");
                } else {
					// clear text pane
                    jobDescPane.setText(""); 
                }
            }
        });
		
		// listen for double-clicks on job nodes
		tree.addMouseListener(new MouseAdapter() 
		{
			public void mousePressed(MouseEvent me) 
			{
				int selRow = tree.getRowForLocation(me.getX(),
													me.getY());
				if (selRow != -1 && me.getClickCount() == 2) 		
				{
					TreePath selPath = tree.getPathForRow(selRow);
					// make sure the path gets selected first
					tree.setSelectionPath(selPath);
					DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                   tree.getLastSelectedPathComponent();

                	if (node == null) return;

                	Object nodeInfo = node.getUserObject();
                	if (node.isLeaf()) {
						if (DEBUG) System.out.println("Job node was double-clicked");
                    	describeJob();
                	}
				}
			}
		});


        //Create the scroll pane and add the tree to it. 
        treeView = new JScrollPane(tree);

        //Create the HTML viewing pane.
        jobDescPane = new JTextArea();
		jobDescPane.setLineWrap(true);
        jobDescPane.setWrapStyleWord(true);
		jobDescPane.setEditable(false);
        JScrollPane descView = new JScrollPane(jobDescPane);

        //Add the scroll panes to a split pane.
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setTopComponent(treeView);
        splitPane.setBottomComponent(descView);

        Dimension minimumSize = new Dimension(100, 50);
        descView.setMinimumSize(minimumSize);
        treeView.setMinimumSize(minimumSize);
        splitPane.setDividerLocation(320); //XXX: ignored in some releases
                                           //of Swing. bug 4101306
        //workaround for bug 4101306:
        //treeView.setPreferredSize(new Dimension(100, 100)); 

        splitPane.setPreferredSize(new Dimension(300, 400));

        //Add the split pane to this frame.
        getContentPane().add(splitPane, BorderLayout.CENTER);
		
		
		//////////////////////////////
		//  add panel with buttons	//
		//////////////////////////////
		buttonPane = new JPanel();
		buttonPane.setLayout(new GridLayout(0,1));
		//Dimension maximumSize = new Dimension(50, 50);
		//buttonPane.setMaximumSize(maximumSize);
		
		addButton(	disableButton, "  1. Disable Job", KeyEvent.VK_D,
					"Click this button to disable the selected job",
					"disable job" );
			
		addButton(	enableButton, "  2. Enable Job", KeyEvent.VK_E, 
					"Click this button to enable the selected job",
					"enable job");
					
		addButton(	logButton, "  3. View Job Log", KeyEvent.VK_L, 
					"Click this button to view event log the selected job",
					"view log");
					
		addButton(	runButton, "  4. Run Job", KeyEvent.VK_R, 
					"Click this button to manually start the selected job",
					"run job");
							
		addButton(	createButton, "  5. Create Job", KeyEvent.VK_C, 
					"Click this button to create a new job",
					"create job");
					
		addButton(	deleteButton, "  6. Delete Job", KeyEvent.VK_T,
					"Click this button to delete the selected job",
					"delete job");
		
		addButton(	modifyButton, "  7. Modify Job", KeyEvent.VK_M,
					"Click this button to modify properties of the selected job",
					"modify job");
					
		addButton(	describeButton, "  8. Describe Job", KeyEvent.VK_B, 
					"Click this button to view properties of the selected job",
					"describe job");
					
		addButton(	commandButton, "  A. Execute Remote Command", KeyEvent.VK_U,
					"Click this button to execute OS command on the remote computer running 24x7 server",
					"execute command");
					
		addButton(	scriptButton, "  B. Execute Remote Script", KeyEvent.VK_S, 
					"Click this button to execute automation script on the remote computer running 24x7 server",
					"execute script");
					
		addButton(	sampleButton, "  C. SQL, FTP, RAS, Telnet, more...", KeyEvent.VK_F, 
					"Click this button to see 24x7 built-in automation functions",
					"show samples");
					
		getContentPane().add(buttonPane, BorderLayout.EAST);
		
		
		show(); // show applet
    }

	
	private void addButton(JButton button,
						  String caption, int mnemonic,
						  String toolTip, String command)
	{
		button = new JButton(caption);
		button.setMnemonic(mnemonic);
		button.setHorizontalAlignment(AbstractButton.LEFT);
		ToolTipManager.sharedInstance().registerComponent(button);
		button.setToolTipText(toolTip);
        button.setActionCommand(command);
		button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				// we have to use hard-coded button "cases"
				// because of the Java limitations for 
				// using "non-final variables"
				if (      e.getActionCommand().equals("disable job") ) 
					disableJob();
				else if ( e.getActionCommand().equals("enable job") ) 
					enableJob();
				else if ( e.getActionCommand().equals("run job") ) 
					runJob();
				else if ( e.getActionCommand().equals("modify job") ) 
					modifyJob();
				else if ( e.getActionCommand().equals("delete job") ) 
					deleteJob();
				else if ( e.getActionCommand().equals("create job") ) 
					createJob();
				else if ( e.getActionCommand().equals("describe job") ) 
					describeJob();
				else if ( e.getActionCommand().equals("view log") ) 
					viewJobLog();
				else if ( e.getActionCommand().equals("execute command") ) 
					executeRemoteCommand();
				else if ( e.getActionCommand().equals("execute script") ) 
					executeRemoteScript();
				else if ( e.getActionCommand().equals("show samples") ) 
					showSupportedFeatures();
					
            }
        });
		buttonPane.add(button);
	} // end addButton
	
	// display message box
	private void showMessage(String msg) 
	{
		JOptionPane.showMessageDialog(this.appletFrame, 
			msg, 
			"Error",
			JOptionPane.WARNING_MESSAGE);
	}
		
		
    private class jobInfo {
		public String jobID;
        public String jobName;
		public String jobDesc;
		public String jobFolderID;
		public String jobFolderName;
		public boolean jobDisabled;
		
        public jobInfo(String id, String name, String desc, 
						boolean disabled, String fid, String fname) {
            jobID = id;
			jobName = name;
			jobDesc = desc; 
			jobDisabled = disabled;
			jobFolderName = fname;
			jobFolderID = fid;
			
			//if (DEBUG) System.out.println("Adding job \"" + jobName  + "\"...");
			
        }
		
		public String toString() {
			return jobName;
		}
	
    } // end class jobInfo

    ///////////////////////////////////////
    //  DISPLAY JOB DESCRIPTION IN BOTTOM PANEL
    ///////////////////////////////////////
	
    private void displayDesc(String jobID) {
		String jobDesc;
		StringHolder buffer = new StringHolder("");
		Integer ReturnValue;
		
		try { 
			if (DEBUG) System.out.println( "Getting description of job #" + jobID + "..." );
			ReturnValue = myRemoteControl.getjobproperty(jobID, 
														"DESCRIPTION", 
														buffer);
			if ( ReturnValue.intValue() == 1 ) {
				jobDesc = (String)buffer.getValue();
				if (jobDesc.equals("")) jobDesc = "Job description not available";
			}
			else {
				// error
				jobDesc = "Job description not available";
			}
			if (DEBUG) System.out.println( "Done" );
		} // end try
		catch(com.sybase.dpb.RemoteException e) {
            showMessage( "RemoteException: " + e.getMessage() );
			jobDesc = "";
        }  // end catch

        jobDescPane.setText(jobDesc);
    }

    ///////////////////////////////////////
    //  CREATE NODES -- POPULATE TREEVIEW
    ///////////////////////////////////////

    private void createNodes(DefaultMutableTreeNode top, String jobList) {
        DefaultMutableTreeNode folder = null;
        DefaultMutableTreeNode job = null;
		String jobID, jobName, jobDisabled, jobFolder, temp = new String("");
		String jobDesc = null;
		
		// parse returned job list
		int startPos = 0, endPos = 0, listLength = 0;
		listLength = jobList.length();

		while (startPos < listLength)
		{
			// get job id
			endPos = jobList.indexOf("\t", startPos);
			jobID = jobList.substring(startPos, endPos);
			startPos = endPos + 1;
			
			// get job name
			endPos = jobList.indexOf("\t", startPos);
			jobName = jobList.substring(startPos, endPos);
			startPos = endPos + 1;
			
			// get job state
			endPos = jobList.indexOf("\t", startPos);
			jobDisabled = jobList.substring(startPos, endPos);
			startPos = endPos + 1;
			
			// get job folder
			endPos = jobList.indexOf("\r\n", startPos);
			jobFolder = jobList.substring(startPos, endPos);
			startPos = endPos + 2;
		
			if (!jobFolder.equals(temp)) 
			{
				// add folder node
	        	folder = new DefaultMutableTreeNode(jobFolder);
	        	top.add(folder);
				temp = jobFolder;
			}

	        // add job node
			job = new DefaultMutableTreeNode(new jobInfo (jobID, jobName, null,
	           						(jobDisabled.equals("Y")), jobID, jobDesc));
	        folder.add(job);
		} // end loop

    } // end createNodes()


    ///////////////////////////////////////
    //  GET LOGIN PARAMETERS DIALOG
    ///////////////////////////////////////

	private boolean getLoginInfo()
	{
		LoginDialog loginDialog = new LoginDialog(this.appletFrame, 
			hostName, portNo.toString(), userID, userPass);
        loginDialog.pack();
		loginDialog.setLocationRelativeTo(this.appletFrame);
        loginDialog.setVisible(true);
		if ( loginDialog.isCanceled()) 
		{
			loginDialog.dispose();
			return false;
		}
		else 
		{
			hostName = loginDialog.getServer();
			portNo = Integer.valueOf(loginDialog.getPort());
			userID = loginDialog.getUser();
			userPass = loginDialog.getPassword();
			loginDialog.dispose();
			return true;
		}
	}

	///////////////////////////////////////
	//  GET SELECTED JOB METHOD
	///////////////////////////////////////
	
	private jobInfo getSelectedJob() 
	{
		DefaultMutableTreeNode node = null;
		jobInfo job = null;
		
		node = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
		if (node != null)
		{
			if (node.getLevel() == 2)
			{ 
				job = (jobInfo)node.getUserObject();
				if (DEBUG) System.out.println("Job #" + job.jobID + " is selected");
				return job;	
			}
		}
		
		if (DEBUG) System.out.println("No job is selected");
		showMessage( "Select the desired job then try again!");
		return null;
	}
	
	///////////////////////////////////////
	//  DISABLE JOB METHOD
	///////////////////////////////////////
	
	private void disableJob() {
		jobInfo job = null;
		Integer ReturnValue;
		
		job = getSelectedJob();
		if (job == null) return;
		
		try { 
			if (DEBUG) System.out.println( "Disabling job #" + job.jobID + "..." );
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			ReturnValue = myRemoteControl.disablejob(job.jobID);
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			if ( ReturnValue.intValue() == 1 ) // success 
			{
				if (DEBUG) System.out.println( "Job disabled" );
				job.jobDisabled = true;
				// update node icon
				tree.repaint();
			}
			else // error
				if (DEBUG) System.out.print( "Cannot disable this job.\n" + 
											myRemoteControl.getLasterror() );
		} // end try
		catch(com.sybase.dpb.RemoteException e) {
            showMessage( "RemoteException: " + e.getMessage() );
        }  // end catch
    }

    ///////////////////////////////////////
    //  ENABLE JOB METHOD
    ///////////////////////////////////////

	private void enableJob() {
		jobInfo job = null;
		Integer ReturnValue;
		
		job = getSelectedJob();
		if (job == null) return;
		
		try { 
			if (DEBUG) System.out.println( "Enabling job #" + job.jobID + "..." );
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			ReturnValue = myRemoteControl.enablejob(job.jobID);
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			if ( ReturnValue.intValue() == 1 ) // success 
			{
				if (DEBUG) System.out.println( "Job enabled" );
				job.jobDisabled = false;
				// update node icon
				tree.repaint();
			}
			else // error
				if (DEBUG) System.out.print( "Cannot enable this job.\n" + 
											myRemoteControl.getLasterror() );
		} // end try
		catch(com.sybase.dpb.RemoteException e) {
            showMessage( "RemoteException: " + e.getMessage() );
        }  // end catch
    }
	

	///////////////////////////////////////
	//  RUN JOB METHOD
	///////////////////////////////////////
	private void runJob() {
		jobInfo job = null;
		Integer ReturnValue;
		
		job = getSelectedJob();
		if (job == null) return;
		
		try { 
			if (DEBUG) System.out.println( "Running job #" + job.jobID + "..." );
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			ReturnValue = myRemoteControl.runjob(job.jobID, new Boolean(false));
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			if ( ReturnValue.intValue() == 1 ) // success 
			{
				if (DEBUG) System.out.println( "Job started" );
			}
			else // error
			{
				if (DEBUG) System.out.print( "Cannot run this job.\n" + 
											myRemoteControl.getLasterror());
			}
		} // end try
		catch(com.sybase.dpb.RemoteException e) {
            showMessage( "RemoteException: " + e.getMessage() );
        }  // end catch
    }


    ///////////////////////////////////////
    //  DELETE JOB METHOD
    ///////////////////////////////////////

    private void deleteJob() {
    	jobInfo job = null;
    	Integer ReturnValue;
    	
    	job = getSelectedJob();
    	if (job == null) return;
    	
		if (JOptionPane.showConfirmDialog(this.appletFrame, 
				"Are you sure you want to delete job \"" + job.jobName + "\"?", 
				"Please Confirm", JOptionPane.YES_NO_OPTION,
				JOptionPane.QUESTION_MESSAGE) != 0) return; // canceled
			
    	try { 
    		if (DEBUG) System.out.println( "Deleting job #" + job.jobID + "..." );
    		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    		ReturnValue = myRemoteControl.deletejob(job.jobID);
    		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    		if ( ReturnValue.intValue() == 1 ) // success 
    		{
    			if (DEBUG) System.out.println( "Job deleted" );
    			// delete node
				DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
    			model.removeNodeFromParent(node);
				jobDescPane.setText(null);
    		}
    		else // error
    			if (DEBUG) System.out.print( "Cannot delete this job.\n" + 
											myRemoteControl.getLasterror() );
    	} // end try
    	catch(com.sybase.dpb.RemoteException e) {
            showMessage( "RemoteException: " + e.getMessage() );
        }  // end catch
    }
    

    ///////////////////////////////////////
    //  DISPLAY NEW JOB DEFINITION DIALOG
	//  AND THEN CREATE NEW JOB
    ///////////////////////////////////////

    private void createJob()
    {
		String jobDefinition;
		Integer ReturnValue;
		
    	CreateJobDialog createDialog = new CreateJobDialog(this.appletFrame);
        createDialog.pack();
    	createDialog.setLocationRelativeTo(this.appletFrame);
        createDialog.setVisible(true);
    	if ( createDialog.isCanceled()) 
    		createDialog.dispose();
    	else 
    	{
    		jobDefinition = createDialog.getJobDefinition();
    		createDialog.dispose();
			// create new job
			try { 
	    		if (DEBUG) System.out.println( "Creating new job..." );
	    		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    		ReturnValue = myRemoteControl.createjob(jobDefinition);
	    		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	    		if ( ReturnValue.intValue() > 1 ) // success 
	    		{
	    			if (DEBUG) System.out.println( "Job created" );
	    			// rebuild treeview
					if (DEBUG) System.out.println( "Refreshing treeview..." );
					setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
					StringHolder jobList = new StringHolder("");
                	ReturnValue =  myRemoteControl.getjoblistex( jobList, new Boolean(false), "DISABLED,FOLDER_NAME" );
	                if (ReturnValue.intValue() == 1) {
						root.removeAllChildren();
	                    // create nodes
	                    createNodes(root, (String)jobList.getValue());
	                } // end if
					
					DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
					model.reload();
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					if (DEBUG) System.out.println( "Done" );
	    		}
	    		else // error
	    			if (DEBUG) System.out.print( "Cannot create new job.\n" + 
											myRemoteControl.getLasterror());
	    	} // end try
	    	catch(com.sybase.dpb.RemoteException e) {
	            showMessage( "RemoteException: " + e.getMessage() );
	        }  // end catch
    	}
    }

    ///////////////////////////////////////
    //  DISPLAY JOB PROPERTIES JOB METHOD
    ///////////////////////////////////////
    
    private void describeJob() {
    	jobInfo job = null;
    	Integer ReturnValue;
		StringHolder buffer = new StringHolder("");
    	
    	job = getSelectedJob();
    	if (job == null) return;
    	
    	try { 
    		if (DEBUG) System.out.println( "Retrieving definition of job #" + job.jobID + "..." );
    		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    		ReturnValue = myRemoteControl.getjobdefinition(job.jobID, 
															buffer,
															new Boolean(true));
    		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			if ( ReturnValue.intValue() == 1 ) {
				if (DEBUG) System.out.println( "Done" );
				// display job properties dialog
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				JobPropertiesView htmlViewer = 
					new JobPropertiesView(this.appletFrame,
										(String)buffer.getValue());
				htmlViewer.pack();
		    	htmlViewer.setLocationRelativeTo(this.appletFrame);
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		        htmlViewer.setVisible(true);
				htmlViewer.dispose();
			}
			else // error
				if (DEBUG) System.out.print( "Error: job definition not available.\n" + 
											myRemoteControl.getLasterror());
    	} // end try
    	catch(com.sybase.dpb.RemoteException e) {
            showMessage( "RemoteException: " + e.getMessage() );
        }  // end catch
    }
	
	
	///////////////////////////////////////
    //  DISPLAY JOB LOG METHOD
    ///////////////////////////////////////
    
    private void viewJobLog() {
    	jobInfo job = null;
    	Integer ReturnValue;
		StringHolder buffer = new StringHolder("");
    	
    	job = getSelectedJob();
    	if (job == null) return;
    	
    	try { 
    		if (DEBUG) System.out.println( "Retrieving log for job #" + job.jobID + "..." );
    		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
    		ReturnValue = myRemoteControl.getjoblog(job.jobID, 
													buffer,
													new Boolean(true));
    		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			if ( ReturnValue.intValue() >= 0 ) {
				if (DEBUG) System.out.println( "Done" );
				// display job properties dialog
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				LogView htmlViewer = 
					new LogView(this.appletFrame, (String)buffer.getValue());
				htmlViewer.pack();
		    	htmlViewer.setLocationRelativeTo(this.appletFrame);
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		        htmlViewer.setVisible(true);
				htmlViewer.dispose();
			}
			else // error
				if (DEBUG) System.out.print( "Error: job log not available.\n" + 
											myRemoteControl.getLasterror());
    	} // end try
    	catch(com.sybase.dpb.RemoteException e) {
            showMessage( "RemoteException: " + e.getMessage() );
        }  // end catch
    }

    ///////////////////////////////////////
    //  MODIFY JOB PROPERTIES METHOD
    ///////////////////////////////////////
    
    private void modifyJob() {
    	jobInfo job = null;
    	Integer ReturnValue;
    	StringHolder buffer = new StringHolder("");
    	
    	job = getSelectedJob();
    	if (job == null) return;
    	
		// display modify job dialog
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		ModifyJobDialog modifyDialog = 
			new ModifyJobDialog(this.appletFrame, job.jobID, job.jobName, myRemoteControl, DEBUG);
		modifyDialog.pack();
    	modifyDialog.setLocationRelativeTo(this.appletFrame);
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        modifyDialog.setVisible(true);
		modifyDialog.dispose();
    }

    ///////////////////////////////////////
    //  EXECUTE REMOTE COMMAND METHOD
    ///////////////////////////////////////
    
    private void executeRemoteCommand() {
    	Integer ReturnValue;
    	String command = new String("");
		boolean async;
		Integer timeout;
		ExecuteCommandDialog commandPrompt = new ExecuteCommandDialog(this.appletFrame);
    	
		// display command dialog
		commandPrompt.pack();
    	commandPrompt.setLocationRelativeTo(this.appletFrame);
        commandPrompt.setVisible(true);
		if ( commandPrompt.isCanceled()) 
    		commandPrompt.dispose();
    	else 
    	{
    		command = commandPrompt.getCommandLine();
			async = commandPrompt.isAsync();
			timeout = commandPrompt.getTimeout();
    		commandPrompt.dispose();
			// execute remote command
			try { 
	    		if (DEBUG) System.out.println( "Executing remote command \"" + command + "\"" );
	    		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    		ReturnValue = myRemoteControl.runshellcommand(command,
															  new String(""),
															  new Boolean(async),
															  timeout);
	    		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	    		if ( ReturnValue.intValue() == 1 ) // success 
				{
	    			if (DEBUG) System.out.println( "Command executed" );
				}
	    		else // error
				{
    				if (DEBUG) System.out.print( "Command failed.\n" + 
    										myRemoteControl.getLasterror());
				}
	    	} // end try
	    	catch(com.sybase.dpb.RemoteException e) {
	            showMessage( "RemoteException: " + e.getMessage() );
	        }  // end catch
    	}
    }


	///////////////////////////////////////
    //  EXECUTE REMOTE JAL SCRIPT METHOD
    ///////////////////////////////////////
    
    private void executeRemoteScript() {
    	Integer ReturnValue;
    	String script = new String("");
		ExecuteScriptDialog scriptPrompt = new ExecuteScriptDialog(this.appletFrame);
    	
		// display script dialog
		scriptPrompt.pack();
    	scriptPrompt.setLocationRelativeTo(this.appletFrame);
        scriptPrompt.setVisible(true);
		if ( scriptPrompt.isCanceled()) 
    		scriptPrompt.dispose();
    	else 
    	{
    		script = scriptPrompt.getScript();
    		scriptPrompt.dispose();
			// execute remote script
			try { 
	    		if (DEBUG) System.out.println( "Executing remote script..." );
	    		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    		ReturnValue = myRemoteControl.runscript(script);
	    		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	    		if ( ReturnValue.intValue() == 1 ) // success 
				{
	    			if (DEBUG) System.out.println( "Script executed" );
				}
	    		else // error
				{
    				if (DEBUG) System.out.print( "Script failed.\n" + 
    										myRemoteControl.getLasterror());
				}
	    	} // end try
	    	catch(com.sybase.dpb.RemoteException e) {
	            showMessage( "RemoteException: " + e.getMessage() );
	        }  // end catch
    	}
    }


    ///////////////////////////////////////
    //  SHOW SUPPORTED FEATURES
    ///////////////////////////////////////
    
    private void showSupportedFeatures() {
		JOptionPane.showConfirmDialog(this.appletFrame, 
				"24x7 Scheduler provides built-in functions for automating a\n" + 
				"wide variete of tasks including SQL, FTP, HTTP, RAS, Telnet,\n" + 
				"Database import/export and replication, File replication,\n" + 
				"File backup, Win NT Event Logs, WinNT Services, and many\n" +
				"other tasks. No other software is required!\n\n" + 
				"To learn more about it, see 24x7 Scheduler manual or see\n" +
				"the on-line help file.", 
				"24x7 Features", JOptionPane.DEFAULT_OPTION,
				JOptionPane.INFORMATION_MESSAGE);
    }

    ///////////////////////////////////////
    //  MAIN
    ///////////////////////////////////////

    public  void init() {
		// find frame
		Container f = getParent();
		while (f != null && !(f instanceof Frame)) 
		{
			f = f.getParent();
		}
		if (f == null) 
		{
			// cannot get frame, terminate applet
			stop();
			return;
		}
		appletFrame = (Frame)f;
		
        
		try {
			//UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} 
		catch(java.lang.Throwable e) {
            showMessage( "Exception: " + e.getMessage() );
        }  

		// connect to the 24x7 server, 
		// retrieve jobs and build job treeview		
        buildTreeView();
        show();
    } // end main()
	

	public void destroy() 
	{
		if (myRemoteControl != null) 
		{
			//   Close session and disconnect from the 
			// 24x7 Scheduler server
			try { 
				if (DEBUG) System.out.println( "Closing session..." );
				myRemoteControl.closesession();
				myConnection.disconnect();
				if (DEBUG) System.out.println( "Disconnected." );
			} // end try
			catch(com.sybase.dpb.RemoteException e) {
	            showMessage( "RemoteException: " + e.getMessage() );
	        }  // end catch
	        
		}
		// Terminate applet
		super.stop();
	}

    ///////////////////////////////////////
    //  TREEVIEW NODE RENDERER CLASS
    ///////////////////////////////////////
	
	private class MyRenderer extends DefaultTreeCellRenderer {
        ImageIcon computerIcon, jobIcon, jobDisabledIcon, folderIcon, folderIcon2;

        public MyRenderer() {
			computerIcon = new ImageIcon("images/computer.gif");
            jobIcon = new ImageIcon("images/job.gif");
			jobDisabledIcon = new ImageIcon("images/disabled.gif");
			folderIcon = new ImageIcon("images/cfolder.gif");
			folderIcon2 = new ImageIcon("images/ofolder.gif");
        } // end MyRenderer()

        public Component getTreeCellRendererComponent(
                            JTree tree,
                            Object value,
                            boolean sel,
                            boolean expanded,
                            boolean leaf,
                            int row,
                            boolean hasFocus) {

            super.getTreeCellRendererComponent(
                            tree, value, sel,
                            expanded, leaf, row,
                            hasFocus);
            if (row > 0 && leaf) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
            	jobInfo nodeInfo = (jobInfo)(node.getUserObject());
            	if ( nodeInfo.jobDisabled ) 
					setIcon(jobDisabledIcon);
				else 
					setIcon(jobIcon);
				
				setToolTipText("Click on the job to see the description");
            } else {
				if (row==0) 
				{
					setIcon(computerIcon);
					setToolTipText("The remote computer running 24x7 Scheduler server");
				}
				else 
				{
					if (expanded) 
						setIcon(folderIcon2);
					else
						setIcon(folderIcon);

                	setToolTipText("Job Folder");
				}
            } // end if

            return this;
        } // end getTreeCellRendererComponent
	} // end class MyRenderer
} // end class MainMenu

